require 'spec_helper'

describe SimpleExcel::Worksheet do
  let(:headers)       { ['ID', 'Value'] }
  let(:file)          { File.join('spec', 'support', 'correct.xls') }
  let(:no_data_file)  { File.join('spec', 'support', 'no_data.xls') }
  let(:two_sheets)    { File.join('spec', 'support', 'two_sheets.xls') }
  let(:simple_excel)  { SimpleExcel.new(file, headers) }
  let(:worksheet)     { SimpleExcel::Worksheet.new(simple_excel.excel, headers) }

  context '.initialize' do
    it 'raise an exception if headers do not match' do
      simple_excel = SimpleExcel.new(no_data_file, ['ID', 'Test'])
      expect{ SimpleExcel::Worksheet.new(simple_excel.excel, simple_excel.headers) }.to raise_error(
        SimpleExcel::Worksheet::HeadersDoNotMatch, "File headers do not coincide with #{['ID', 'Test']}"
      )
    end

    it 'check headers and return sheet' do
      simple_excel  = SimpleExcel.new(two_sheets, headers)
      worksheet     = SimpleExcel::Worksheet.new(simple_excel.excel, simple_excel.headers)

      sheet_index = 0
      simple_excel.excel.worksheets.each_with_index do |sheet, index|
        if simple_excel.excel.worksheet(index).row(0) == headers
          sheet_index = index
        end
      end

      worksheet.sheet.should_not == simple_excel.excel.worksheet(0)
      worksheet.sheet.should == simple_excel.excel.worksheet(sheet_index)
      worksheet.sheet.should be_an_instance_of Spreadsheet::Excel::Worksheet
    end

    it 'returns first sheet if check_headers argument set to false' do
      worksheet = SimpleExcel::Worksheet.new(simple_excel.excel, simple_excel.headers, false)
      worksheet.sheet.should be_an_instance_of Spreadsheet::Excel::Worksheet
    end

    it 'returns instance of SimpleExcel::Worksheet' do
      worksheet.should be_an_instance_of SimpleExcel::Worksheet
      worksheet.excel.should be_an_instance_of Spreadsheet::Excel::Workbook
      worksheet.headers.should be_an_instance_of Array
      worksheet.sheet.should be_an_instance_of Spreadsheet::Excel::Worksheet
    end
  end

  context '#to_s' do
    it 'returns name of class' do
      worksheet.to_s.should eq worksheet.class
    end
  end

  context '#each' do
    it 'raise an exception if excel file contains no data' do
      excel = SimpleExcel.new(no_data_file, headers)
      worksheet = SimpleExcel::Worksheet.new(excel.excel, excel.headers)
      expect{ worksheet.each }.to raise_error(
        SimpleExcel::Worksheet::FileWithoutData, 'File contains no data'
      )
    end

    it 'returns row' do
      row = worksheet.send(:get_values, 1)
      expect { |block| worksheet.each(&block) }.to yield_successive_args(row)
    end
  end

  context '#no_data?' do
    it 'return true if excel file contains no data' do
      excel = SimpleExcel.new(no_data_file, headers)
      worksheet = SimpleExcel::Worksheet.new(excel.excel, excel.headers)
      worksheet.send(:no_data?).should be_true
    end

    it 'return false if excel file with data' do
      worksheet.send(:no_data?).should be_false
    end
  end

  context '#get_values' do
    it 'return hash of values with headers' do
      row = worksheet.send(:get_values, 1)
      row.should be_an_instance_of Hash
      row.should include('ID' => 1, 'Value' => 'Test', 'index' => 1)
    end
  end
end