require 'spec_helper'

describe SimpleExcel do
  let(:headers)                   { ['ID', 'Value'] }
  let(:invalid_url)               { 'http://invalid.url' }

  let(:correct_file)              { File.join('spec', 'support', 'correct.xls') }
  let(:no_data_file)              { File.join('spec', 'support', 'no_data.xls') }
  let(:not_found_file)            { File.join('spec', 'support', 'not_found') }
  let(:not_supported_format_file) { File.join('spec', 'support', 'not_supported_format.ods') }

  let(:simple_excel)              { SimpleExcel.new(correct_file, headers) }

  context '::EXTENSIONS_AVAILABLE' do
    it 'returns array of available extensions' do
      SimpleExcel::EXTENSIONS_AVAILABLE.should be_an_instance_of Array
      SimpleExcel::EXTENSIONS_AVAILABLE.should == ['xls']
    end
  end

  context '.initialize' do
    it 'raise an exception if calling method without arguments' do
      expect{ SimpleExcel.new }.to raise_error(
        ArgumentError, 'wrong number of arguments (0 for 2)'
      )
    end

    it 'raise an exception if first argument is blank' do
      expect{ SimpleExcel.new('', '') }.to raise_error(
        ArgumentError, 'Path to file or url a blank'
      )
    end

    it 'raise an exception if url is invalid' do
      expect{ SimpleExcel.new(invalid_url, '') }.to raise_error(
        SimpleExcel::InvalidUrl, "Invalid URL: #{invalid_url}"
      )
    end

    it 'raise an exception if file not found' do
      expect{ SimpleExcel.new(not_found_file, '') }.to raise_error(
        SimpleExcel::FileNotFound, "File '#{not_found_file}' not found"
      )
    end

    it 'raise an exception if file does not open' do
      expect{ SimpleExcel.new(not_supported_format_file, '') }.to raise_error(
        SimpleExcel::UnsupportedExcelFile, "File '#{not_supported_format_file}' not supported"
      )
    end

    it 'raise an exception if headers is blank' do
      expect{ SimpleExcel.new(correct_file, '') }.to raise_error(
        SimpleExcel::RequiredHeaders, 'Headers is required'
      )
    end

    it 'raise and exception if second argument exists, but is not an Array' do
      expect{ SimpleExcel.new(correct_file, 'ID, VALUE') }.to raise_error(
        SimpleExcel::HeadersMustBeAnArray, 'Headers are not an array'
      )
    end

    it 'should set @excel' do
      simple_excel.excel.should be_an_instance_of Spreadsheet::Excel::Workbook
    end

    it 'should set @headers' do
      simple_excel.headers.should be_an_instance_of(Array)
      simple_excel.headers.should eq headers
    end

    it 'open excel file and return instance of SimpleExcel' do
      simple_excel.should be_an_instance_of SimpleExcel
    end
  end

  context '#to_s' do
    it 'returns name of class' do
      simple_excel.to_s.should eq simple_excel.class
    end
  end

  context '#each' do
    it 'raise an exception if excel file contains no data' do
      simple_excel = SimpleExcel.new(no_data_file, headers)
      expect{ simple_excel.each }.to raise_error(
        SimpleExcel::Worksheet::FileWithoutData, 'File contains no data'
      )
    end

    it 'returns row' do
      worksheet = SimpleExcel::Worksheet.new(simple_excel.excel, simple_excel.headers)
      row = worksheet.send(:get_values, 1)
      expect { |block| simple_excel.each(&block) }.to yield_successive_args(row)
    end
  end

  context '#checking_fill_fields' do
    it 'raise an exception if exclude argument is not Array' do
      worksheet = SimpleExcel::Worksheet.new(simple_excel.excel, simple_excel.headers)
      row = worksheet.send(:get_values, 1)
      expect{ simple_excel.send(:checking_fill_fields, row, 'test') }.to raise_error(
        SimpleExcel::ExcludeMustBeAnArray, 'Headers are not an array'
      )
    end

    it 'raise an exception if field is not filled' do
      simple_excel = SimpleExcel.new(no_data_file, headers)
      worksheet = SimpleExcel::Worksheet.new(simple_excel.excel, simple_excel.headers)
      row = worksheet.send(:get_values, 1)
      expect{ simple_excel.send(:checking_fill_fields, row) }.to raise_error(
        SimpleExcel::NotFilledField, "Field '#{headers.first}' is not filled"
      )
    end
  end

  context '#save_to_output' do
    it 'returns instance of SimpleExcel::ExtStringIO' do
      simple_excel.save_to_output(correct_file).should be_an_instance_of SimpleExcel::ExtStringIO
    end
  end
end